#!/usr/pkg/bin/perl

$| = 1;

use MIME::Base64;
use POSIX qw/ceil/;
use Crypt::CBC;
use Getopt::Long;

sub get_rndctl_stats {
    my($rndctl);
    my(@rndctl);
    $rndctl = `/sbin/rndctl -s`;
    @rndctl = split("\n",$rndctl);
    my($x);
    my($c);
    my($m);
    foreach $x (@rndctl) {
	if ($x =~ /^[ 	]*([0-9]*) bits currently stored in pool .max ([0-9]*)./) {
	    $c = $1;
	    $m = $2;
	    last;
	}
    }

    return($c,$m);
}

my($key_file);
my($random_device) = "/dev/random";
my($log);

GetOptions("key-file=s" => \$key_file,
	   "log" => \$log,
	   "random-device=s" => \$random_device
    ) or die "Argument error\n";

my(%keys);
if (defined $key_file) {
    if ( -s "$key_file") {
	my($key);
	my($id);
	my($salt);
	open(KEYFILE,$key_file) || die "Could not open keyfile $key_file: $!";
	while (<KEYFILE>) {
	    chomp;
	    if (/^#/) {
		next;
	    }
	    if (/^$/) {
		next;
	    }
	    if (/^key:/i) {
		my($k,$v) = split(/:/);
		$key = $v;
	    }
	    if (/^id:/i) {
		my($k,$v) = split(/:/);
		$id = $v;
		undef $key;
		undef $salt;
		undef $iv
	    }
	    if (/^salt:/i) {
		my($k,$v) = split(/:/);
		$salt = $v;
	    }
	    if (defined $id) {
		if (defined $key) {
		    $keys{$id}{'key'} = $key;
		}
		if (defined $salt) {
		    $keys{$id}{'salt'} = $salt;
		}
	    }
	}
	close KEYFILE;
    }
}

my($id)="Unknown";
my($want_encryption);
my($state);
my($ggot)=0;

print "200 - Hello\n";
$state = "RUNNING";

while (<STDIN>) {
    chomp;
    my($command);
    my(@args);

    if ($_ =~ / /) {
	($command,@args) = split(/ /);
    } else {
	$command = $_;
    }

    $command = uc($command);

    if ($state eq "RUNNING") {
	if ($command eq "GET") {
	    my($want);
	    $want = $args[0];
	    if ($want ne "") {
		if ($want > 0) {
		    my($current);
		    my($max);
		    ($current,$max) = get_rndctl_stats();
		    if ($current + 0.0 > 128) {
			if ($want <= $current) {
			    my($bits);
			    my($got);
			    my($wantbytes);
			    $wantbytes = ceil($want / 8);
			    sysopen RANDOM,$random_device,O_RDONLY || die "Could not open randomness: $!\n";
			    while ($wantbytes > 0) {
				my($b);
				my($s);
				if ($wantbytes > 16) {
				    $s = 16;
				} else {
				    $s = $wantbytes;
				}
				$got = sysread(RANDOM,$b,$s);
				if ($got > 0) {
				    $bits .= $b;
				    $wantbytes -= $got;
				} else {
				    last;
				}
			    }
			    close RANDOM;

			    # open(DEBUG,"> /tmp/DEBUG");
			    # print DEBUG $bits;
			    # close DEBUG;
			    
			    my($enc);
			    if (defined $want_encryption) {
				my(%cargs);
				$cargs{'-key'} = $keys{$want_encryption}{'key'};
				$cargs{'-cipher'} = "Crypt::OpenSSL::AES";
				if (defined $keys{$want_encruption}{'salt'}) {
				    $cargs{'-salt'} = $keys{$want_encruption}{'salt'};
				}
				my($cipher) = Crypt::CBC->new(%cargs);
				# my($cipher) = Crypt::CBC->new(-key => $keys{$want_encryption}{'key'},
				# 			      -cipher => "Crypt::OpenSSL::AES"
				#     );
				# my($cipher) = Crypt::CBC->new(-key => $key,
				# 			      -cipher => "Crypt::Rijndael"
				#     );

				my($encrypted) = $cipher->encrypt($bits);
				$bits = $encrypted;
			    }
			    $enc = encode_base64($bits,"");
			    print "100 - $enc\n";
			    $ggot++;
			    if (defined $log) {
				my($m);
				$m = "/usr/bin/logger -p local6.info " . '"' . "Sent " . length($bits) . " bytes with pid $$ $ggot times for $id" . '"';
				system($m);
			    }
			} else {
			    print "402 - Not enough bits available\n";
			    if (defined $log) {
				my($m);
				$m = "/usr/bin/logger -p local6.info " . '"' . "402 Not enough bits available for pid $$ for $id" . '"';
				system($m);
			    }
			}
		    } else {
			print "401 - Not enough bits available\n";
			if (defined $log) {
			    my($m);
			    $m = "/usr/bin/logger -p local6.info " . '"' . "401 Not enough bits available for pid $$ for $id" . '"';
			    system($m);
			}
		    }
		} else {
		    print "403 - amount must be greater than 0\n";
		}
	    } else {
		print "404 - need to know amount\n";
	    }
	    next;
	}
	if ($command eq "ID") {
	    my($i) = $args[0];
	    if ($i ne "") {
		$id = $i;
		print "203 - Nice to meet you $id\n";
	    } else {
		print "406 - Who are you...\n";
	    }
	}
	if ($command eq "ENCRYPT") {
	    my($keyid) = $args[0];
	    if (defined $keys{$keyid}{'key'}) {
		$want_encryption = $keyid;
		print "202 - Encrypting\n";
	    } else {
		print "405 - No key defined for $keyid\n";
	    }
	}
	if ($command eq "BYE") {
	    print "201 - Bye\n";
	    $state = "ENDING";
	    exit;
	}
    }
}
